/*
 * ble_whts.h
 *
 *  Created on: Dec 16, 2015
 *      Author: C9917732
 */

#ifndef BLE_WHTS_H__
#define BLE_WHTS_H__

#include <stdint.h>
#include <stdbool.h>
#include "ble.h"
#include "ble_srv_common.h"

#define WHTS_BASE_UUID {{ 0x1b, 0xc5, 0xd5, 0xa5, 0x02, 0x00, 0x78, 0xa3, \
        0xe5, 0x11, 0xac, 0xb8, 0xa0, 0x49, 0x18, 0x02 } }
#define BLE_UUID_WHTS_SERVICE 0x0001     /**< The UUID of the custom service          */
#define BLE_UUID_WHTS_SENSOR  0x0002     /**< The UUID of the sensor characteristic   */
#define BLE_UUID_WHTS_LED     0x0003     /**< The UUID of the LED ctrl characteristic */


/** @brief Water Heater Service event type */
typedef enum
{
    BLE_WHTS_EVT_NOTIFICATION_ENABLED,  /**< WHT value notification enabled event  */
    BLE_WHTS_EVT_NOTIFICATION_DISABLED, /**< WHT value notification disabled event */
    BLE_WHTS_EVT_SET_LED_CFG,           /**< set LED cfg event                     */
} ble_whts_evt_type_t;

typedef struct
{
    ble_whts_evt_type_t evt_type;       /**< type of event */
    union
    {
        uint16_t led_cfg;               /**< the led cfg value */
    } params;
} ble_whts_evt_t;

// Forward declaration of the ble_whts_t type
typedef struct ble_whts_s ble_whts_t;

/**@brief Water Heater Temperature Service event handler type. */
typedef void (*ble_whts_evt_handler_t) (ble_whts_t* p_whts, ble_whts_evt_t* p_evt);

/**@brief Water Heater Temperature Service init structure. This contains all options and data
 *        needed for initialization of the service. */
typedef struct
{
    ble_whts_evt_handler_t       evt_handler;          /**< Event handler to be called for whts. */
    bool                         support_notification; /**< TRUE if notification of temp sensor measurement is supported. */
    uint16_t                     initial_temp_level;   /**< Initial temperature level */
    ble_srv_cccd_security_mode_t whts_temp_sensor_attr_md; /**< initial security lvl for sensor      */

} ble_whts_init_t;

/**@brief Water Heater Temperature Service structure. This contains various status information for
 *        the service. */
struct ble_whts_s
{
    ble_whts_evt_handler_t       evt_handler;    /**< Event handler to be called for whts.     */
    uint16_t                     service_handle; /**< Handle of whts provided by stack         */
    ble_gatts_char_handles_t     sensor_handles; /**< handles for temp sensor measurement char. */
    ble_gatts_char_handles_t     led_handles;    /**< handles for the led characteristic       */
    uint8_t                      uuid_type;      /**< UUID type for water heater temp svc      */
    uint16_t                     temp_value_last; /**< last value                              */
    uint16_t                     conn_handle;    /**< Handle of the current connection (as provided by the BLE stack, is BLE_CONN_HANDLE_INVALID if not in a connection). */
    bool                         is_notification_supported; /**< TRUE if notification of temp value is supported */
};

/**@brief Function for handling the Application's BLE Stack events.
 *
 * @details Handles all events from the BLE stack of interest to the water heater temp svc.
 *
 * @note
 *
 * @param[in]   p_whts     Water Heater Temp Svc structure.
 * @param[in]   p_ble_evt  Event received from the BLE stack.
 */
void ble_whts_on_ble_evt(ble_whts_t * p_whts, ble_evt_t * p_ble_evt);

/**@brief Function for initializing the Water Heater Temperature Service.
 *
 * @param[out]  p_whts      Water Heater Temperature Service structure. This structure will have to
 *                          be supplied by the application. It will be initialized by this function,
 *                          and will later be used to identify this particular service instance.
 * @param[in]   p_whts_init Information needed to initialize the service.
 *
 * @return      NRF_SUCCESS on successful initialization of service, otherwise an error code.
 */
uint32_t ble_whts_init(ble_whts_t* p_whts, const ble_whts_init_t* p_whts_init);

/**@brief Function for updating the temp sensor level.
 *
 * @details The application calls this function after having performed a temp sensor measurement.
 *          If notification has been enabled, the temp sesor characteristic is sent to the
 *          client.
 *
 * @note For the requirements in the WHTS specification to be fulfilled,
 *       this function must be called upon reconnection if the tmep sensor has changed
 *       while the service has been disconnected from a bonded client.
 *
 * @param[in]   p_whts         Water Htr Temp Service structure.
 * @param[in]   temp_val       New temp sensor measurement value (in tenths of a degree).
 *
 * @return      NRF_SUCCESS on success, otherwise an error code.
 */
uint32_t ble_whts_temp_sensor_update(ble_whts_t * p_whts, uint16_t temp_val);


#endif
